@@ -8,16 +8,16 @@
 
 static const char* FILE_NAME = "Net/SSLeay/EVP_PKEY.c";
 
-int32_t SPVM__Net__SSLeay__X509__DESTROY(SPVM_ENV* env, SPVM_VALUE* stack) {
+int32_t SPVM__Net__SSLeay__EVP_PKEY__DESTROY(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   int32_t error_id = 0;
   
   void* obj_self = stack[0].oval;
   
-  X509* x509 = env->get_pointer(env, stack, obj_self);
+  EVP_PKEY* evp_pkey = env->get_pointer(env, stack, obj_self);
   
   if (!env->no_free(env, stack, obj_self)) {
-    X509_free(x509);
+    EVP_PKEY_free(evp_pkey);
   }
   
   return 0;
